@@ -7,17 +7,21 @@
 // notice may not be copied, modified, or distributed except
 // according to those terms.
 
-use std::sync::Arc;
+use std::{convert::TryFrom, sync::Arc};
 
+use bytemuck::{Pod, Zeroable};
 use egui::{ScrollArea, TextEdit, TextStyle};
 use egui_winit_vulkano::Gui;
 use vulkano::{
     buffer::{BufferUsage, CpuAccessibleBuffer, TypedBufferAccess},
     command_buffer::{AutoCommandBufferBuilder, CommandBufferUsage, SubpassContents},
-    device::{physical::PhysicalDevice, Device, DeviceExtensions, Features, Queue},
+    device::{
+        physical::PhysicalDevice, Device, DeviceCreateInfo, DeviceExtensions, Features, Queue,
+        QueueCreateInfo,
+    },
     format::Format,
-    image::{view::ImageView, ImageAccess, ImageUsage, SwapchainImage},
-    instance::{Instance, InstanceExtensions},
+    image::{view::ImageView, ImageAccess, ImageUsage, ImageViewAbstract, SwapchainImage},
+    instance::{Instance, InstanceCreateInfo, InstanceExtensions},
     pipeline::{
         graphics::{
             input_assembly::InputAssemblyState,
@@ -26,11 +30,10 @@ use vulkano::{
         },
         GraphicsPipeline,
     },
-    render_pass::{Framebuffer, RenderPass, Subpass},
+    render_pass::{Framebuffer, FramebufferCreateInfo, RenderPass, Subpass},
     swapchain,
     swapchain::{
-        AcquireError, ColorSpace, FullscreenExclusive, PresentMode, Surface, SurfaceTransform,
-        Swapchain, SwapchainCreationError,
+        AcquireError, PresentMode, Surface, Swapchain, SwapchainCreateInfo, SwapchainCreationError,
     },
     sync,
     sync::{FlushError, GpuFuture},
@@ -55,21 +58,25 @@ pub fn main() {
     // Create gui state (pass anything your state requires)
     let mut code = CODE.to_owned();
     event_loop.run(move |event, _, control_flow| {
-        // Update Egui integration so the UI works!
-        gui.update(&event);
         match event {
-            Event::WindowEvent { event, window_id } if window_id == window_id => match event {
-                WindowEvent::Resized(_) => {
-                    renderer.resize();
+            Event::WindowEvent { event, window_id }
+                if window_id == renderer.surface().window().id() =>
+            {
+                // Update Egui integration so the UI works!
+                let _pass_events_to_game = !gui.update(&event);
+                match event {
+                    WindowEvent::Resized(_) => {
+                        renderer.resize();
+                    }
+                    WindowEvent::ScaleFactorChanged { .. } => {
+                        renderer.resize();
+                    }
+                    WindowEvent::CloseRequested => {
+                        *control_flow = ControlFlow::Exit;
+                    }
+                    _ => (),
                 }
-                WindowEvent::ScaleFactorChanged { .. } => {
-                    renderer.resize();
-                }
-                WindowEvent::CloseRequested => {
-                    *control_flow = ControlFlow::Exit;
-                }
-                _ => (),
-            },
+            }
             Event::RedrawRequested(window_id) if window_id == window_id => {
                 // Set immediate UI in redraw here
                 gui.immediate_ui(|gui| {
@@ -84,8 +91,7 @@ pub fn main() {
                                 &mut columns[0],
                                 |ui| {
                                     ui.add(
-                                        TextEdit::multiline(&mut code)
-                                            .text_style(TextStyle::Monospace),
+                                        TextEdit::multiline(&mut code).font(TextStyle::Monospace),
                                     );
                                 },
                             );
@@ -143,8 +149,12 @@ impl SimpleGuiRenderer {
         // Add instance extensions based on needs
         let instance_extensions = InstanceExtensions { ..vulkano_win::required_extensions() };
         // Create instance
-        let instance = Instance::new(None, Version::V1_2, &instance_extensions, None)
-            .expect("Failed to create instance");
+        let instance = Instance::new(InstanceCreateInfo {
+            application_version: Version::V1_2,
+            enabled_extensions: instance_extensions,
+            ..Default::default()
+        })
+        .expect("Failed to create instance");
         // Get most performant device (physical)
         let physical = PhysicalDevice::enumerate(&instance)
             .fold(None, |acc, val| {
@@ -169,15 +179,10 @@ impl SimpleGuiRenderer {
         // Create device
         let (device, queue) = Self::create_device(physical, surface.clone());
         // Create swap chain & frame(s) to which we'll render
-        let (swap_chain, images) = Self::create_swap_chain(
-            surface.clone(),
-            physical,
-            device.clone(),
-            queue.clone(),
-            present_mode,
-        );
+        let (swap_chain, images) =
+            Self::create_swap_chain(surface.clone(), physical, device.clone(), present_mode);
         let previous_frame_end = Some(sync::now(device.clone()).boxed());
-        let render_pass = Self::create_render_pass(device.clone(), swap_chain.format());
+        let render_pass = Self::create_render_pass(device.clone(), images[0].format().unwrap());
         let pipeline = Self::create_pipeline(device.clone(), render_pass.clone());
 
         let vertex_buffer = {
@@ -218,7 +223,7 @@ impl SimpleGuiRenderer {
     ) -> (Arc<Device>, Arc<Queue>) {
         let queue_family = physical
             .queue_families()
-            .find(|&q| q.supports_graphics() && surface.is_supported(q).unwrap_or(false))
+            .find(|&q| q.supports_graphics() && q.supports_surface(&surface).unwrap_or(false))
             .expect("couldn't find a graphical queue family");
         // Add device extensions based on needs
         let device_extensions =
@@ -226,12 +231,12 @@ impl SimpleGuiRenderer {
         // Add device features
         let features = Features::none();
         let (device, mut queues) = {
-            Device::new(
-                physical,
-                &features,
-                &physical.required_extensions().union(&device_extensions),
-                [(queue_family, 0.5)].iter().cloned(),
-            )
+            Device::new(physical, DeviceCreateInfo {
+                enabled_extensions: physical.required_extensions().union(&device_extensions),
+                enabled_features: features,
+                queue_create_infos: vec![QueueCreateInfo::family(queue_family)],
+                _ne: Default::default(),
+            })
             .expect("failed to create device")
         };
         (device, queues.next().unwrap())
@@ -241,31 +246,29 @@ impl SimpleGuiRenderer {
         surface: Arc<Surface<Window>>,
         physical: PhysicalDevice,
         device: Arc<Device>,
-        queue: Arc<Queue>,
         present_mode: PresentMode,
     ) -> (Arc<Swapchain<Window>>, Vec<Arc<ImageView<SwapchainImage<Window>>>>) {
-        let caps = surface.capabilities(physical).unwrap();
-        let alpha = caps.supported_composite_alpha.iter().next().unwrap();
-        let format = caps.supported_formats[0].0;
-        let dimensions: [u32; 2] = surface.window().inner_size().into();
-        let (swap_chain, images) = Swapchain::start(device, surface)
-            .num_images(caps.min_image_count)
-            .format(format)
-            .dimensions(dimensions)
-            .usage(ImageUsage::color_attachment())
-            .sharing_mode(&queue)
-            .composite_alpha(alpha)
-            .transform(SurfaceTransform::Identity)
-            .present_mode(present_mode)
-            .fullscreen_exclusive(FullscreenExclusive::Default)
-            .clipped(true)
-            .color_space(ColorSpace::SrgbNonLinear)
-            .layers(1)
-            .build()
-            .unwrap();
-        let images =
-            images.into_iter().map(|image| ImageView::new(image).unwrap()).collect::<Vec<_>>();
-        (swap_chain, images)
+        let surface_capabilities =
+            physical.surface_capabilities(&surface, Default::default()).unwrap();
+        let image_format =
+            Some(physical.surface_formats(&surface, Default::default()).unwrap()[0].0);
+        let image_extent = surface.window().inner_size().into();
+
+        let (swapchain, images) = Swapchain::new(device, surface, SwapchainCreateInfo {
+            min_image_count: surface_capabilities.min_image_count,
+            image_format,
+            image_extent,
+            image_usage: ImageUsage::color_attachment(),
+            composite_alpha: surface_capabilities.supported_composite_alpha.iter().next().unwrap(),
+            present_mode,
+            ..Default::default()
+        })
+        .unwrap();
+        let images = images
+            .into_iter()
+            .map(|image| ImageView::new_default(image).unwrap())
+            .collect::<Vec<_>>();
+        (swapchain, images)
     }
 
     fn create_render_pass(device: Arc<Device>, format: Format) -> Arc<RenderPass> {
@@ -345,11 +348,11 @@ impl SimpleGuiRenderer {
         .unwrap();
 
         let dimensions = self.final_images[0].image().dimensions().width_height();
-        let framebuffer = Framebuffer::start(self.render_pass.clone())
-            .add(self.final_images[image_num].clone())
-            .unwrap()
-            .build()
-            .unwrap();
+        let framebuffer = Framebuffer::new(self.render_pass.clone(), FramebufferCreateInfo {
+            attachments: vec![self.final_images[image_num].clone()],
+            ..Default::default()
+        })
+        .unwrap();
 
         // Begin render pipeline commands
         let clear_values = vec![[0.0, 1.0, 0.0, 1.0].into()];
@@ -395,23 +398,26 @@ impl SimpleGuiRenderer {
         let after_future = before_future.then_execute(self.queue.clone(), command_buffer).unwrap();
 
         // Finish render
-        self.finish(Box::new(after_future), image_num);
+        self.finish(after_future.boxed(), image_num);
     }
 
     fn recreate_swapchain(&mut self) {
         let dimensions: [u32; 2] = self.surface.window().inner_size().into();
-        let (new_swapchain, new_images) =
-            match self.swap_chain.recreate().dimensions(dimensions).build() {
-                Ok(r) => r,
-                Err(SwapchainCreationError::UnsupportedDimensions) => return,
-                Err(e) => panic!("Failed to recreate swapchain: {:?}", e),
-            };
+        let (new_swapchain, new_images) = match self.swap_chain.recreate(SwapchainCreateInfo {
+            image_extent: dimensions,
+            ..self.swap_chain.create_info()
+        }) {
+            Ok(r) => r,
+            Err(SwapchainCreationError::ImageExtentNotSupported { .. }) => return,
+            Err(e) => panic!("Failed to recreate swapchain: {:?}", e),
+        };
         self.swap_chain = new_swapchain;
-        let new_images =
-            new_images.into_iter().map(|image| ImageView::new(image).unwrap()).collect::<Vec<_>>();
+        let new_images = new_images
+            .into_iter()
+            .map(|image| ImageView::new_default(image).unwrap())
+            .collect::<Vec<_>>();
         self.final_images = new_images;
-        // self.framebuffers =
-        //     framebuffer_from_swapchain_images(&self.final_images, self.render_pass.clone());
+
         self.recreate_swapchain = false;
     }
 
@@ -441,7 +447,8 @@ impl SimpleGuiRenderer {
     }
 }
 
-#[derive(Default, Debug, Clone)]
+#[repr(C)]
+#[derive(Default, Debug, Copy, Clone, Zeroable, Pod)]
 struct Vertex {
     position: [f32; 2],
     color: [f32; 4],
