@@ -7,11 +7,10 @@
 // notice may not be copied, modified, or distributed except
 // according to those terms.
 
-mod context;
-mod conversions;
 mod integration;
 mod renderer;
 mod utils;
 
+pub use egui;
 pub use integration::*;
 pub use utils::{texture_from_bytes, texture_from_file};
